/*
  $Id: AllowedCharacterRule.java 2022 2011-07-01 22:21:57Z dfisher $

  Copyright (C) 2003-2011 Virginia Tech.
  All rights reserved.

  SEE LICENSE FOR MORE INFORMATION

  Author:  Middleware Services
  Email:   middleware@vt.edu
  Version: $Revision: 2022 $
  Updated: $Date: 2011-07-01 18:21:57 -0400 (Fri, 01 Jul 2011) $
*/
package edu.vt.middleware.password;

import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Rule for determining if a password contains allowed characters.
 *
 * @author  Middleware Services
 * @version  $Revision: 2022 $ $Date: 2011-07-01 18:21:57 -0400 (Fri, 01 Jul 2011) $
 */
public class AllowedCharacterRule implements Rule
{

  /** Error code for allowed character failures. */
  public static final String ERROR_CODE = "ALLOWED_CHAR";

  /** Stores the characters that are allowed. */
  private char[] allowedChar;


  /**
   * Create a new allowed character rule.
   *
   * @param  c  allowed characters
   */
  public AllowedCharacterRule(final char[] c)
  {
    allowedChar = c;
    Arrays.sort(allowedChar);
  }


  /** {@inheritDoc} */
  @Override
  public RuleResult validate(final PasswordData passwordData)
  {
    final RuleResult result = new RuleResult(true);

    for (char c : passwordData.getPassword().getText().toCharArray()) {
      if (Arrays.binarySearch(allowedChar, c) < 0) {
        result.setValid(false);
        result.getDetails().add(
          new RuleResultDetail(
            ERROR_CODE, createRuleResultDetailParameters(c)));
        break;
      }
    }
    return result;
  }


  /**
   * Creates the parameter data for the rule result detail.
   *
   * @param  c  illegal character
   *
   * @return  map of parameter name to value
   */
  protected Map<String, ?> createRuleResultDetailParameters(final char c)
  {
    final Map<String, Object> m = new LinkedHashMap<String, Object>();
    m.put("illegalCharacter", c);
    return m;
  }


  /**
   * Returns a string representation of this object.
   *
   * @return  string representation
   */
  @Override
  public String toString()
  {
    return
      String.format(
        "%s@%h::allowedChar=%s",
        getClass().getName(),
        hashCode(),
        allowedChar != null ? Arrays.asList(allowedChar) : null);
  }
}
